@@ -37,14 +37,15 @@ from typing import Tuple, Union, List, Dict
 # pySMART module imports
 from .attribute import Attribute
 from .diagnostics import Diagnostics
+from .interface import *
+from .smartctl import Smartctl, SMARTCTL
 from .testentry import TestEntry
-from .smartctl import Smartctl
 from .utils import smartctl_type, smartctl_isvalid_type, any_in, all_in
 
 logger = logging.getLogger('pySMART')
 
 
-def smart_health_assement(disk_name, smartctl: Smartctl = Smartctl()):
+def smart_health_assement(disk_name, smartctl: Smartctl = SMARTCTL):
     """
     This function gets the SMART Health Status of the disk (IF the disk
     is SMART capable and smart is enabled on it else returns None).
@@ -75,7 +76,7 @@ class Device(object):
     (considered SATA) but excludes other external devices (USB, Firewire).
     """
 
-    def __init__(self, name: str, interface: str = None, abridged: bool = False, smart_options: Union[str, List[str]] = None, smartctl: Smartctl = Smartctl()):
+    def __init__(self, name: str, interface: str = None, abridged: bool = False, smart_options: Union[str, List[str]] = None, smartctl: Smartctl = SMARTCTL):
         """Instantiates and initializes the `pySMART.device.Device`."""
         if not (
                 interface is None or
@@ -100,6 +101,8 @@ class Device(object):
         """**(str):** Device's model number."""
         self.serial: str = None
         """**(str):** Device's serial number."""
+        self.vendor: str = None
+        """**(str):** Device's vendor (if any)."""
         self.interface: str = None if interface == 'UNKNOWN INTERFACE' else interface
         """
         **(str):** Device's interface type. Must be one of:
@@ -205,13 +208,29 @@ class Device(object):
         we need to grep/regex the shit outta the normal "smartctl -a" output.
         In case the device have more than one temperature sensor the first value
         will be stored here too.
+        Note: Temperatures are always in Celsius (if possible).
         """
-        self.temperatures = {}
+        self.temperatures: Dict[int, int] = {}
         """
         **(dict of int): NVMe disks usually report multiple temperatures, which
         will be stored here if available. Keys are sensor numbers as reported in
         output data.
+        Note: Temperatures are always in Celsius (if possible).
         """
+        self.logical_sector_size: int = None
+        """
+        **(int):** The logical sector size of the device (or LBA).
+        """
+        self.physical_sector_size: int = None
+        """
+        **(int):** The physical sector size of the device.
+        """
+        self.if_attributes: Union[None, NvmeAttributes] = None
+        """
+        **(NvmeAttributes):** This object may vary for each device interface attributes.
+        It will store all data obtained from smartctl
+        """
+
         if self.name is None:
             warnings.warn(
                 "\nDevice '{0}' does not exist! This object should be destroyed.".format(
@@ -225,7 +244,7 @@ class Device(object):
             logger.trace(
                 "Determining interface of disk: {0}".format(self.name))
             raw, returncode = self.smartctl.generic_call(
-                ['-d', 'test', os.path.join('/dev/', self.name)])
+                ['-d', 'test', self.dev_reference])
 
             if len(raw) > 0:
                 # I do not like this parsing logic but it works for now!
@@ -263,6 +282,24 @@ class Device(object):
         if self.interface is not None or self.abridged:
             self.update()
 
+    @property
+    def dev_reference(self) -> str:
+        """The reference to the device as provided by smartctl.
+           - On unix-like systems, this is the path to the device. (example /dev/<name>)
+           - On MacOS, this is the name of the device. (example <name>)
+           - On Windows, this is the drive letter of the device. (example <drive letter>)
+
+        Returns:
+            str: The reference to the device as provided by smartctl.
+        """
+
+        # detect if we are on MacOS
+        if 'IOService' in self.name:
+            return self.name
+
+        # otherwise asume we are on unix-like systems
+        return os.path.join('/dev/', self.name)
+
     @property
     def capacity(self) -> str:
         """Returns the capacity in the raw smartctl format.
@@ -299,6 +336,20 @@ class Device(object):
 
         return humanfriendly.parse_size(self._capacity)
 
+    @property
+    def sector_size(self) -> int:
+        """Returns the sector size of the device.
+
+        Returns:
+            int: The sector size of the device in Bytes. If undefined, we'll assume 512B
+        """
+        if self.logical_sector_size is not None:
+            return self.logical_sector_size
+        elif self.physical_sector_size is not None:
+            return self.physical_sector_size
+        else:
+            return 512
+
     def __repr__(self):
         """Define a basic representation of the class object."""
         return "<{0} device on /dev/{1} mod:{2} sn:{3}>".format(
@@ -330,7 +381,7 @@ class Device(object):
         if all_info:
             state_dict.update({
                 'name': self.name,
-                'path': os.path.join('/dev/', self.name),
+                'path': self.dev_reference,
                 'serial': self.serial,
                 'is_ssd': self.is_ssd,
                 'rotation_rate': self.rotation_rate,
@@ -369,7 +420,7 @@ class Device(object):
             if action_lower == 'off':
                 return True, None
         raw, returncode = self.smartctl.generic_call(
-            ['-s', action_lower, os.path.join('/dev/', self.name)])
+            ['-s', action_lower, self.dev_reference])
 
         if returncode != 0:
             return False, raw
@@ -447,7 +498,7 @@ class Device(object):
                 smartctl_type(test),
                 '-l',
                 'sataphy',
-                os.path.join('/dev/', self.name)])
+                self.dev_reference])
 
             if 'GP Log 0x11' in raw[3]:
                 self.interface = test
@@ -459,14 +510,14 @@ class Device(object):
                 'scsi',
                 '-l',
                 'sasphy',
-                os.path.join('/dev/', self.name)])
+                self.dev_reference])
             if 'SAS SSP' in raw[4]:
                 self.interface = 'sas'
             # Some older SAS devices do not support the SAS PHY log command.
             # For these, see if smartmontools reports a transport protocol.
             else:
                 raw = self.smartctl.all(
-                    'scsi', os.path.join('/dev/', self.name))
+                    'scsi', self.dev_reference)
 
                 for line in raw:
                     if 'Transport protocol' in line and 'SAS' in line:
@@ -589,7 +640,7 @@ class Device(object):
         # Returns:
         * **(int):** The returncode of calling `smartctl -X device_path`
         """
-        return self.smartctl.test_stop(smartctl_type(self.interface), os.path.join('/dev/', self.name))
+        return self.smartctl.test_stop(smartctl_type(self.interface), self.dev_reference)
 
     def run_selftest(self, test_type, ETA_type='date'):
         """
@@ -654,7 +705,7 @@ class Device(object):
             return 2, "Unknown test type '{0}' requested.".format(test_type), None
 
         raw = self.smartctl.test_start(
-            interface, test_type, os.path.join('/dev/', self.name))
+            interface, test_type, self.dev_reference)
         _success = False
         _running = False
         for line in raw:
@@ -777,12 +828,12 @@ class Device(object):
         self.temperatures = {}
         if self.abridged:
             interface = None
-            raw = self.smartctl.info(os.path.join('/dev/', self.name))
+            raw = self.smartctl.info(self.dev_reference)
 
         else:
             interface = smartctl_type(self.interface)
             raw = self.smartctl.all(
-                interface, os.path.join('/dev/', self.name))
+                interface, self.dev_reference)
 
         parse_self_tests = False
         parse_running_test = False
@@ -793,7 +844,21 @@ class Device(object):
         self._test_progress = None
         # Lets skip the first couple of non-useful lines
         _stdout = raw[4:]
-        for line in _stdout:
+
+        #######################################
+        #   Dedicated interface attributes    #
+        #######################################
+
+        if interface == 'nvme':
+            self.if_attributes = NvmeAttributes(iter(_stdout))
+        else:
+            self.if_attributes = None
+
+        #######################################
+        #    Global / generic  attributes     #
+        #######################################
+        stdout_iter = iter(_stdout)
+        for line in stdout_iter:
             if line.strip() == '':  # Blank line stops sub-captures
                 if parse_self_tests is True:
                     parse_self_tests = False
@@ -866,20 +931,31 @@ class Device(object):
                 self.model = line.split(':')[1].lstrip().rstrip()
                 self._guess_smart_type(line.lower())
                 continue
+
             if 'Model Family' in line:
                 self._guess_smart_type(line.lower())
                 continue
+
             if 'LU WWN' in line:
                 self._guess_smart_type(line.lower())
                 continue
+
             if any_in(line, 'Serial Number', 'Serial number'):
                 self.serial = line.split(':')[1].split()[0].rstrip()
                 continue
+
+            vendor = re.compile(r'^Vendor:\s+(\w+)').match(line)
+            if vendor is not None:
+                self.vendor = vendor.groups()[0]
+
             if any_in(line, 'Firmware Version', 'Revision'):
                 self.firmware = line.split(':')[1].strip()
+
             if any_in(line, 'User Capacity', 'Namespace 1 Size/Capacity'):
                 # TODO: support for multiple NVMe namespaces
-                self._capacity = line.replace(']', '[').split('[')[1].strip()
+                self._capacity = line.replace(']', '[').split('[')[
+                    1].strip().replace(',', '.')
+
             if 'SMART support' in line:
                 # self.smart_capable = 'Available' in line
                 # self.smart_enabled = 'Enabled' in line
@@ -896,10 +972,12 @@ class Device(object):
                 elif any_in(line, 'Available', 'device has SMART capability'):
                     self.smart_capable = True
                 continue
+
             if 'does not support SMART' in line:
                 self.smart_capable = False
                 self.smart_enabled = False
                 continue
+
             if 'Rotation Rate' in line:
                 if 'Solid State Device' in line:
                     self.is_ssd = True
@@ -918,6 +996,8 @@ class Device(object):
                     self.assessment = 'PASS'
                 else:
                     self.assessment = 'FAIL'
+                continue
+
             if 'SMART Health Status' in line:  # SCSI devices
                 if line.split(':')[1].strip() == 'OK':
                     self.assessment = 'PASS'
@@ -925,19 +1005,27 @@ class Device(object):
                     self.assessment = 'FAIL'
                     parse_ascq = True  # Set flag to capture status message
                     message = line.split(':')[1].lstrip().rstrip()
+                continue
+
             # Parse SMART test capabilities (ATA only)
             # Note: SCSI does not list this but and allows for only 'offline', 'short' and 'long'
             if 'SMART execute Offline immediate' in line:
                 self.test_capabilities['offline'] = 'No' not in line
-            if 'Self-test supported' in line:
-                self.test_capabilities['short'] = 'No' not in line
-                self.test_capabilities['short'] = 'No' not in line
+                continue
+
             if 'Conveyance Self-test supported' in line:
                 self.test_capabilities['conveyance'] = 'No' not in line
-            # Note: Currently I have not added any support in pySMART for selective Self-tests
-            # Thus commenting it out
-            # if 'Selective Self-test supported' in line:
-            #     self.test_capabilities['selective'] = False if 'No' in line else True
+                continue
+
+            if 'Selective Self-test supported' in line:
+                self.test_capabilities['selective'] = 'No' not in line
+                continue
+
+            if 'Self-test supported' in line:
+                self.test_capabilities['short'] = 'No' not in line
+                self.test_capabilities['long'] = 'No' not in line
+                continue
+
             # SMART Attribute table parsing
             if all_in(line, '0x0', '_') and not interface == 'nvme':
                 # Replace multiple space separators with a single space, then
@@ -955,7 +1043,6 @@ class Device(object):
                     # for ATA the "%" remaining is on the next line
                     # thus set the parse_running_test flag and move on
                     parse_running_test = True
-                    continue
                 elif '%' in line:
                     # for scsi the progress is on the same line
                     # so we can just parse it and move on
@@ -965,6 +1052,7 @@ class Device(object):
                             int(line.split('%')[0][-3:].strip())
                     except ValueError:
                         pass
+                continue
             if parse_running_test is True:
                 try:
                     self._test_progress = 100 - \
@@ -972,16 +1060,26 @@ class Device(object):
                 except ValueError:
                     pass
                 parse_running_test = False
+
             if all_in(line, 'Description', '(hours)'):
                 parse_self_tests = True  # Set flag to capture test entries
+
+            #######################################
+            #              SCSI only              #
+            #######################################
+            #
             # Everything from here on is parsing SCSI information that takes
             # the place of similar ATA SMART information
             if 'used endurance' in line:
                 pct = int(line.split(':')[1].strip()[:-1])
                 self.diagnostics.Life_Left = 100 - pct
+                continue
+
             if 'Specified cycle count' in line:
                 self.diagnostics.Start_Stop_Spec = int(
                     line.split(':')[1].strip())
+                continue
+
             if 'Accumulated start-stop cycles' in line:
                 self.diagnostics.Start_Stop_Cycles = int(
                     line.split(':')[1].strip())
@@ -989,9 +1087,13 @@ class Device(object):
                     self.diagnostics.Start_Stop_Pct_Left = int(round(
                         100 - (self.diagnostics.Start_Stop_Cycles /
                                self.diagnostics.Start_Stop_Spec), 0))
+                continue
+
             if 'Specified load-unload count' in line:
                 self.diagnostics.Load_Cycle_Spec = int(
                     line.split(':')[1].strip())
+                continue
+
             if 'Accumulated load-unload cycles' in line:
                 self.diagnostics.Load_Cycle_Count = int(
                     line.split(':')[1].strip())
@@ -999,9 +1101,13 @@ class Device(object):
                     self.diagnostics.Load_Cycle_Pct_Left = int(round(
                         100 - (self.diagnostics.Load_Cycle_Count /
                                self.diagnostics.Load_Cycle_Spec), 0))
+                continue
+
             if 'Elements in grown defect list' in line:
                 self.diagnostics.Reallocated_Sector_Ct = int(
                     line.split(':')[1].strip())
+                continue
+
             if 'read:' in line:
                 line_ = ' '.join(line.split()).split(' ')
                 if line_[1] == '0' and line_[2] == '0' and line_[3] == '0' and line_[4] == '0':
@@ -1011,8 +1117,10 @@ class Device(object):
                         line_[1]) + int(line_[2]) + int(line_[3])
                 else:
                     self.diagnostics.Corrected_Reads = int(line_[4])
-                self.diagnostics.Reads_GB = float(line_[6])
-                self.diagnostics.Uncorrected_Reads = int(line_[7])
+                self.diagnostics._Reads_GB = float(line_[6].replace(',', '.'))
+                self.diagnostics._Uncorrected_Reads = int(line_[7])
+                continue
+
             if 'write:' in line:
                 line_ = ' '.join(line.split()).split(' ')
                 if (line_[1] == '0' and line_[2] == '0' and
@@ -1023,8 +1131,10 @@ class Device(object):
                         line_[1]) + int(line_[2]) + int(line_[3])
                 else:
                     self.diagnostics.Corrected_Writes = int(line_[4])
-                self.diagnostics.Writes_GB = float(line_[6])
-                self.diagnostics.Uncorrected_Writes = int(line_[7])
+                self.diagnostics._Writes_GB = float(line_[6].replace(',', '.'))
+                self.diagnostics._Uncorrected_Writes = int(line_[7])
+                continue
+
             if 'verify:' in line:
                 line_ = ' '.join(line.split()).split(' ')
                 if (line_[1] == '0' and line_[2] == '0' and
@@ -1035,21 +1145,35 @@ class Device(object):
                         line_[1]) + int(line_[2]) + int(line_[3])
                 else:
                     self.diagnostics.Corrected_Verifies = int(line_[4])
-                self.diagnostics.Verifies_GB = float(line_[6])
-                self.diagnostics.Uncorrected_Verifies = int(line_[7])
+                self.diagnostics._Verifies_GB = float(
+                    line_[6].replace(',', '.'))
+                self.diagnostics._Uncorrected_Verifies = int(line_[7])
+                continue
+
             if 'non-medium error count' in line:
                 self.diagnostics.Non_Medium_Errors = int(
                     line.split(':')[1].strip())
+                continue
+
             if 'Accumulated power on time' in line:
                 self.diagnostics.Power_On_Hours = int(
                     line.split(':')[1].split(' ')[1])
+                continue
+
             if 'Current Drive Temperature' in line or ('Temperature:' in
                                                        line and interface == 'nvme'):
                 try:
                     self.temperature = int(
                         line.split(':')[-1].strip().split()[0])
+
+                    if 'fahrenheit' in line.lower():
+                        self.temperature = int((self.temperature - 32) * 5 / 9)
+
                 except ValueError:
                     pass
+
+                continue
+
             if 'Temperature Sensor ' in line:
                 try:
                     match = re.search(
@@ -1058,11 +1182,49 @@ class Device(object):
                         (tempsensor_number_s, tempsensor_value_s) = match.group(1, 2)
                         tempsensor_number = int(tempsensor_number_s)
                         tempsensor_value = int(tempsensor_value_s)
+
+                        if 'fahrenheit' in line.lower():
+                            tempsensor_value = int(
+                                (tempsensor_value - 32) * 5 / 9)
+
                         self.temperatures[tempsensor_number] = tempsensor_value
                         if self.temperature is None or tempsensor_number == 0:
                             self.temperature = tempsensor_value
                 except ValueError:
                     pass
+
+                continue
+
+            #######################################
+            #            Common values            #
+            #######################################
+
+            # Sector sizes
+            if 'Sector Sizes' in line:  # ATA
+                m = re.match(
+                    r'.* (\d+) bytes logical,\s*(\d+) bytes physical', line)
+                if m:
+                    self.logical_sector_size = int(m.group(1))
+                    self.physical_sector_size = int(m.group(2))
+                    # set diagnostics block size to physical sector size
+                    self.diagnostics._block_size = self.physical_sector_size
+                continue
+            if 'Logical block size:' in line:  # SCSI 1/2
+                self.logical_sector_size = int(
+                    line.split(':')[1].strip().split(' ')[0])
+                # set diagnostics block size to logical sector size
+                self.diagnostics._block_size = self.logical_sector_size
+                continue
+            if 'Physical block size:' in line:  # SCSI 2/2
+                self.physical_sector_size = int(
+                    line.split(':')[1].strip().split(' ')[0])
+                continue
+            if 'Namespace 1 Formatted LBA Size' in line:  # NVMe
+                # Note: we will assume that there is only one namespace
+                self.logical_sector_size = int(
+                    line.split(':')[1].strip().split(' ')[0])
+                continue
+
         if not self.abridged:
             if not interface == 'scsi':
                 # Parse the SMART table for below-threshold attributes and create
@@ -1078,7 +1240,7 @@ class Device(object):
                             'scsi',
                             '-l',
                             'background',
-                            os.path.join('/dev/', self.name)
+                            self.dev_reference
                         ])
 
                     for line in raw:
